<?xml version="1.0" encoding="UTF-8"?>
<!--
    SectionContainer Structure Transform
    B. Collier Jones (bcollierjones@gmail.com)
    
    This structure converts the default folder/portlet layout structure stored
    in the uPortal database into a more skinning-friendly XML format.  Also,
    this transform facilitates navigation within the portal by enforcing rules
    as to which nodes in a layout nodes can be selected for navigation.
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
    <!-- This is ignored by uPortal, but included for development -->
    <xsl:output method="xml" indent="yes"/>
    
    <!-- Parameter: nodeID - ID of node to make active -->
    <xsl:param name="nodeID"/>
    
    <!-- Parameter: renderMode - The rendering mode of the portal.
        Known modes include:
        * "view" - normal viewing of portal content (default)
        * "edit" - additional layout modification hooks are rendered
        * "detached" - a portlet is being rendered in detached mode
        * "ajax" - a portlet is being rendered dynamically via AJAX
    -->
    <xsl:param name="renderMode">
        <xsl:choose>
            <!-- When: A @mode is included with the layout -->
            <xsl:when test="/portalContent/@mode">
                <xsl:value-of select="/portalContent/@mode"/>
            </xsl:when>
            <!-- Otherwise: Assume we are in view mode -->
            <xsl:otherwise>view</xsl:otherwise>
        </xsl:choose>
    </xsl:param>            

    <!-- Key: layoutNodes - Used to quickly access nodes in a layout without
        using expensive // queries -->
    <xsl:key name="layoutNodes" match="folder|portlet" use="@ID"/>
    
    <!-- Variable: defaultSectionName - Name of the section to display if no
        section is specificially request.  (Usually on user login) -->
    <xsl:variable name="defaultSectionName">home</xsl:variable>
    
    <!-- Variable: activeNodeID - The node in the layout that is active.  If
        an invalid node is requested, a valid node is chosen.  In the worst
        case, the default section is chosen. -->
    <xsl:variable name="activeNodeID">
        <xsl:choose>
            <!-- When: The node request exists and is not the 'root' node,
                use the requested nodeID -->
            <xsl:when test="(key('layoutNodes', $nodeID)) and ($nodeID != 'root')">
                <xsl:value-of select="$nodeID"/>
            </xsl:when>
            <!-- When: The renderMode is ajax or detached, use the ID of
                the portlet being rendered -->
            <xsl:when test="($renderMode = 'ajax') or ($renderMode = 'detached')">
                <xsl:value-of select="/portalContent/portlet/@ID"/>
            </xsl:when>
            <!-- When: There is a section named $defaultSectionName, use that section -->
            <xsl:when test="//folder[(@name = $defaultSectionName) and (attribute[(@name = 'folderType') and (@provider = 'structure')]/@value = 'section')]/@ID">
                <xsl:value-of select="//folder[(@name = $defaultSectionName) and (attribute[(@name = 'folderType') and (@provider = 'structure')]/@value = 'section')]/@ID"/>
            </xsl:when>
            <!-- NOTE: The following conditions are failover conditions and
                should not occur under normal conditions -->
            <!-- When:  There is a visible first level folder that does not
                have portlets as its childen (meaning its a root level
                container), choose the first one -->
            <xsl:when test="/portalContent/layout/folder[@ID = 'root']/folder[(count(portlet) = 0) and not(@hidden = 'true')]">
                <xsl:value-of select="/portalContent/layout/folder[@ID = 'root']/folder[(count(portlet) = 0) and not(@hidden = 'true')][1]/@ID"/>
            </xsl:when>
            <!-- Otherwise: Choose the first root level folder -->
            <xsl:otherwise>
                <xsl:value-of select="/portalContent/layout/folder[@ID = 'root']/folder[1]/@ID"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:variable>
    
    <!-- Template: portalContent -->
    <xsl:template match="portalContent">
        <portalContent activeNodeID="{$activeNodeID}" renderMode="{$renderMode}">
            <!-- Insert messages here if we want them -->
            <xsl:apply-templates/>
        </portalContent>
    </xsl:template>
    
    <!-- Template: portalContent/portlet -->
    <xsl:template match="portalContent/portlet">
        <portlet>
            <xsl:copy-of select="@*"/>
            <xsl:copy-of select="*" />
        </portlet>
    </xsl:template>
    
    <!-- Template: layout -->
    <xsl:template match="layout">
        <layout ID="root">
            <xsl:apply-templates select="folder[@ID = 'root']/folder"/>
        </layout>
    </xsl:template>
    
    <!-- Template: folder -->
    <xsl:template match="folder">
        <xsl:variable name="folderType" select="attribute[(@name = 'folderType') and (@provider = 'structure')]/@value"/>
        <xsl:variable name="elementName">
            <xsl:choose>
                <!-- When: The folder type is specified, use it -->
                <xsl:when test="$folderType = 'section'">section</xsl:when>
                <xsl:when test="$folderType = 'container'">container</xsl:when>
                <!-- NOTE: The following scenarios are for when there is
                    incomplete layout information. (like by importing a layout
                    from a previous version of uPortal) -->
                <!-- When: The folder is one level deep from the layout
                    element and has folders as its childen it is a section -->
                <xsl:when test="(count(ancestor::folder) = 1) and (count(folder) &gt; 0)">section</xsl:when>
                <!-- When: The folder is one level deep from the layout
                    element and has portlets as its childen it is a container -->
                <xsl:when test="(count(ancestor::folder) = 1) and (count(portlet) &gt; 0)">container</xsl:when>
                <!-- When: The folder is one level deep from the layout
                    element and has no childen it is a section -->
                <xsl:when test="(count(ancestor::folder) = 1) and (count(descendant::folder) = 0) and (count(descendant::portlet) = 0)">section</xsl:when>
                <!-- Otherwise: The folder is a container -->
                <xsl:otherwise>container</xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        
        <xsl:element name="{$elementName}">
            <xsl:copy-of select="@*"/>
            <xsl:apply-templates select="attribute"/>
            <xsl:apply-templates select="portlet"/>
            <xsl:apply-templates select="folder"/>
        </xsl:element>
    </xsl:template>
    
    <!-- Template: attribute -->
    <xsl:template match="attribute">
        <xsl:choose>
            <!-- Strip out 'structure' attributes -->
            <xsl:when test="@provider = 'structure'"/>
            <xsl:otherwise>
                <xsl:copy-of select="."/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Template: portlet -->
    <xsl:template match="portlet">
        <portlet>
            <xsl:copy-of select="@*"/>
            <xsl:apply-templates select="attribute"/>
        </portlet>
    </xsl:template>
    
    <!-- Template text()
        Normalizes whitespace in the input -->
    <xsl:template match="text()">
        <xsl:value-of select="normalize-space()"/>
    </xsl:template>
</xsl:stylesheet>
