<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
    <!-- This is ignored by uPortal, but included for development -->
    <xsl:output method="xml" indent="yes"/>
    <!-- Key: layoutNodes
         A key of all folders and portlets so we don't have to do // searches -->
    <xsl:key name="layoutNodes" match="folder|portlet" use="@ID"/>
    
    <!-- Variable: defaultTabName
         The name of the tab to display when none has been specified -->
    <xsl:variable name="defaultTabName">home</xsl:variable>
    
    <!-- Variable: defaultPageName
         The name of the page to display when none has been specified -->
    <xsl:variable name="defaultPageName">default</xsl:variable>
    
    <!-- Parameter: nodeID
         The ID of the node (folder or portlet) that is currently active.
         Default Value: none -->
    <xsl:param name="nodeID"></xsl:param>
    
    <!-- Variable: validNodeID
         Verifies that the node passed is correct: page, menu, or portlet -->
    <xsl:variable name="validNodeID">
        <xsl:choose>
            <!-- When: nodeID is a tab
                 Then: Use the default page of the tab specified, if it exists -->
            <xsl:when test="(key('layoutNodes', $nodeID)/attribute[@name = 'folderType']/@value = 'tab')">
                <xsl:choose>
                    <!-- When: The default page exists on the tab
                         Then: Use the default page -->
                    <xsl:when test="(key('layoutNodes', $nodeID)/folder[(@name = $defaultPageName) and (attribute[(@name = 'folderType') and (@provider = 'structure')]/@value = 'page')])">
                        <xsl:value-of select="key('layoutNodes', $nodeID)/folder[(@name = $defaultPageName) and (attribute[(@name = 'folderType') and (@provider = 'structure')]/@value = 'page')]/@ID"/>
                    </xsl:when>
                    <!-- Otherwise: Use the first page on the tab -->
                    <xsl:otherwise>
                        <xsl:value-of select="key('layoutNodes', $nodeID)/folder[1]/@ID"/>
                    </xsl:otherwise>
                </xsl:choose> 
            </xsl:when>
            <!-- When: nodeID is a page
                 Then: Use the nodeID -->
            <xsl:when test="(key('layoutNodes', $nodeID)/attribute[(@name = 'folderType') and (@provider = 'structure')]/@value = 'page')">
                <xsl:value-of select="$nodeID"/>
            </xsl:when>
            <!-- When: nodeID is a menu
                 Then: Use the nodeID -->
            <xsl:when test="(key('layoutNodes', $nodeID)/attribute[(@name = 'folderType') and (@provider = 'structure')]/@value = 'menu')">
                <!--<xsl:value-of select="$nodeID"/>-->
                <xsl:value-of select="key('layoutNodes', $nodeID)/descendant::portlet[1]/@ID"/>
            </xsl:when>
            <!-- When: nodeID is a portlet
                 Then: Use the nodeID -->
            <xsl:when test="(name(key('layoutNodes', $nodeID)) = 'portlet')">
                <xsl:value-of select="$nodeID"/>
            </xsl:when>
            <!-- Otherwise: Use the default tab's default page -->
            <xsl:otherwise>
                <!-- try to determine if the specified node may be a tab or a page (but without folderType attribute) -->
                <xsl:choose>
                    <!-- is it a tab ? -->
                    <xsl:when test="(count(key('layoutNodes', $nodeID)/ancestor-or-self::folder)=2) and (key('layoutNodes', $nodeID)/folder)">
                        <xsl:value-of select="folder[1]/@ID"/>
                    </xsl:when>
                    <!-- is it a page ? -->
                    <xsl:when test="key('layoutNodes', $nodeID)/folder">
                        <xsl:value-of select="$nodeID"/>
                    </xsl:when>
                    <!-- try to select default by name -->
                    <xsl:when test="//folder[(attribute[(@name = 'folderType') and (@provider = 'structure')]/@value = 'tab') and (@name = $defaultTabName)]/folder[(attribute[(@name = 'folderType') and (@provider = 'structure')]/@value = 'page') and (@name = $defaultPageName)]">
                        <xsl:value-of select="//folder[(attribute[(@name = 'folderType') and (@provider = 'structure')]/@value = 'tab') and (@name = $defaultTabName)]/folder[(attribute[(@name = 'folderType') and (@provider = 'structure')]/@value = 'page') and (@name = $defaultPageName)]/@ID"/>
                    </xsl:when>
                    <!-- select first top-level folder -->
                    <xsl:otherwise>
                        <xsl:value-of select="//layout/folder/folder[1]/@ID"/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:variable>
    
    <!-- Parameter: mode
         The current mode of the portal view.
         Default Value: 'view' -->
    <xsl:param name="mode">
        <xsl:choose>
            <!-- TODO: modes: edit, detatched, ajax -->
            <xsl:when test="(/portalContent/@mode)"><xsl:value-of select="/portalContent/@mode"/></xsl:when>
            <xsl:when test="(key('layoutNodes', $validNodeID)/ancestor-or-self::folder[attribute[@name = 'folderType']/@value = 'menu'])">menu</xsl:when>
            <xsl:when test="(name(key('layoutNodes', $validNodeID)) = 'portlet')">focused</xsl:when>
            <xsl:otherwise>view</xsl:otherwise>
        </xsl:choose>
    </xsl:param>
    
    <!-- Template: portalContent -->
    <xsl:template match="portalContent">
        <portalContent activeNodeID="{$validNodeID}" mode="{$mode}">
            <xsl:choose>
                <xsl:when test="($nodeID = '')">
                    <message level="structure" code="500">No node requested, going to default tab and default page.</message>
                </xsl:when>
                <xsl:when test="(not(key('layoutNodes', $nodeID)))">
                    <message level="structure" code="501">Invalid node requested, going to default tab and default page.</message>
                </xsl:when>
                <xsl:when test="($nodeID != $validNodeID)">
                    <message level="structure" code="502">The requested node was invalid and adjusted to an appropriate node. Requested: <xsl:value-of select="$nodeID"/>, Adjusted: <xsl:value-of select="$validNodeID"/></message>
                </xsl:when>
            </xsl:choose>
            <xsl:apply-templates/>
        </portalContent>
    </xsl:template>
    
    <!-- Template: message
         Simply copy these for the theme transform to process -->
    <xsl:template match="message">
        <xsl:copy-of select="."/>
    </xsl:template>
    
    <!-- Template: portalContent/portlet -->
    <xsl:template match="portalContent/portlet">
        <portlet>
            <xsl:copy-of select="@*"/>
            <xsl:copy-of select="*" />
        </portlet>
    </xsl:template>
    
    <!-- Template: layout -->
    <xsl:template match="layout">
        <layout>
            <!-- Layouts are automatically included inside of root folders -->
            <xsl:for-each select="folder">
                <xsl:apply-templates/>
           </xsl:for-each>
        </layout>
    </xsl:template>
    
    <!-- Template: root folder -->
    <xsl:template match="folder[@ID = 'root']">
        <folder ID="root">
            <xsl:apply-templates/>
        </folder>
    </xsl:template>
    
    <!-- Template: folder -->
    <xsl:template match="folder">
        
        <!-- Variable: folderType
             Convenience variable for the current folder's folderType attribute -->
        <xsl:variable name="folderType">
            <xsl:choose>
                <!-- When: folderType attribute is specified
                     Then: Use the folderType attribute -->
                <xsl:when test="(attribute[(@name = 'folderType') and (@provider = 'structure')]/@value)">
                    <xsl:value-of select="attribute[(@name = 'folderType') and (@provider = 'structure')]/@value"/>
                </xsl:when>
                <!-- When: folder has an attribute 'hidden' set to 'true'
                     Then: folder is a folder -->
                <xsl:when test="(@hidden = 'true')">folder</xsl:when>
                <!-- When: folder element is two levels deep from the layout element
                     Then: folder is a tab -->
                <xsl:when test="(count(ancestor-or-self::folder) = 2)">tab</xsl:when>
                <!-- When: folder doesn't contain any other folders
                     When: folder is a column -->
                <xsl:when test="(count(descendant-or-self::folder) = 1)">column</xsl:when>
                <!-- Otherwise: folder is a page -->
                <xsl:otherwise>page</xsl:otherwise>
            </xsl:choose>
        </xsl:variable> 

        <!-- Variable: isActiveFolder
             Should the portlets inside this folder be rendered? Yes, in two cases:
             1. The folder is a column at the top level of the layout (headers, footers)
             2. The folder is a column under the active page -->
        <xsl:variable name="isActiveFolder">
            <xsl:choose>
                <!-- When: folder is hidden.   Then: false -->
                <xsl:when test="(@hidden = 'true')">false</xsl:when>
                <!-- When: Folder is a root level header, footer, or column -->
                <xsl:when test="((($folderType = 'header') or ($folderType = 'footer') or ($folderType = 'column')) and (name(../..) = 'layout'))">true</xsl:when>
                <xsl:when test="ancestor-or-self::folder[@ID = $validNodeID]">true</xsl:when>
                <!--<xsl:when test="((../@ID = $validNodeID) and (../attribute[(@name = 'folderType') and (@provider = 'structure')]/@value = 'page'))">true</xsl:when>-->
                <xsl:otherwise>false</xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        
        <!-- Create an element named after the folderType attribute -->
        <xsl:element name="{$folderType}">
            <xsl:copy-of select="@*"/>
            <xsl:apply-templates select="attribute"/>
            <xsl:apply-templates select="portlet">
                <xsl:with-param name="isActiveFolder">
                    <xsl:value-of select="$isActiveFolder"/>
                </xsl:with-param>
            </xsl:apply-templates>
            <xsl:apply-templates select="folder"/>
        </xsl:element>
    </xsl:template>
    
    <!-- Template: attribute -->
    <xsl:template match="attribute">
        <xsl:choose>
            <!-- Strip out all of the 'structure' attributes -->
            <!--<xsl:when test="(@provider = 'structure')"/>-->
            <xsl:when test="(@provider = 'debug')"/>
            <xsl:otherwise>
                <xsl:copy-of select="."/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Template: portlet -->
    <xsl:template match="portlet">
        <xsl:param name="isActiveFolder">false</xsl:param>
        <xsl:choose>
            <!-- When: This portlet is the specific portlet requested (focused)
                 Then: Render the portlet -->
            <xsl:when test="@ID = $validNodeID">
                <xsl:copy>
                    <xsl:copy-of select="@*"/>
                    <xsl:copy-of select="*"/>
                </xsl:copy>
            </xsl:when>
            <!-- When: This portlet is in an active folder
                 Then: Render the portlet -->
            <xsl:when test="$isActiveFolder = 'true'">
                <xsl:copy>
                    <xsl:copy-of select="@*"/>
                    <xsl:copy-of select="*"/>
                </xsl:copy>
            </xsl:when>
            <!-- Otherwise: Don't render the portlet, but include all of its information for other uses -->
            <xsl:otherwise>
                <xsl:element name="inactivePortlet">
                    <xsl:copy-of select="@*"/>
                    <xsl:copy-of select="*"/>
                </xsl:element>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Template text()
         Normalizes whitespace in the input -->
    <xsl:template match="text()">
        <xsl:value-of select="normalize-space()"/>
    </xsl:template>
</xsl:stylesheet>
